/*
 * Copyright (c) 2005 DocJava, Inc. All Rights Reserved.
 */
package gui.run;


import gui.ClosableJFrame;
import gui.layouts.DialogLayout;
import j2d.ImageUtils;
import j3d.polyViewer.vec3.*;
import j3d.utils.PolyhedralShape3d;
import j3d.utils.RunTexCoordGenerationPanel;
import j3d.utils.TriangleGeometryUtils;
import j3d.utils.Utils;
import utils.ResourceManager;

import javax.media.j3d.Appearance;
import javax.media.j3d.Material;
import javax.media.j3d.Shape3D;
import javax.media.j3d.TexCoordGeneration;
import javax.swing.*;
import javax.vecmath.Color3f;
import java.awt.*;
import java.awt.event.MouseEvent;
import java.awt.event.MouseListener;
import java.awt.event.MouseMotionListener;


public abstract class RunShapePanel
        extends JTabbedPane implements Runnable,
        MouseMotionListener,
        MouseListener {
    Vec3FunctionInterface shapesFunctionInterface[] = {
        new TorusVec3FunctionInterface(),
        new MobiusVec3FunctionInterface(),
        new KleinVec3FunctionInterface(),
        new GaussVec3FunctionInterface(),
        new DerivativeOfGaussVec3FunctionInterface(),
        new LaplacianOfGaussVec3FunctionInterface(),
        new MessedUpVec3FunctionInterface(),
        new GaborCosVec3FunctionInterface(),
        new GaborSinVec3FunctionInterface(),
        new ParabolicCynFunctionInterface(),
        new j3d.polyViewer.vec3.ConeVec3FunctionInterface(),
        new j3d.polyViewer.vec3.EllipsoidVec3FunctionInterface(),
        new j3d.polyViewer.vec3.ParabolicCylinderVec3FunctionInterface(),
        new j3d.polyViewer.vec3.RealEllipticalCylinderInterface(),
        new j3d.polyViewer.vec3.HyperbolicParaboloid(),
        new j3d.polyViewer.vec3.EllipticParaboloid(),

    };
    Vec3FunctionInterface currentShape = shapesFunctionInterface[0];
    // # divisions around equator
    private SpinnerNumberModel n1 = new SpinnerNumberModel(3,
            3,
            200,
            1);
    // # divisions through hole
    private SpinnerNumberModel n2 = new SpinnerNumberModel(2,
            2,
            200,
            1);
    // mid-radius   (0 to 1)
    private SpinnerNumberModel r1 = new SpinnerNumberModel(22,
            0,
            1000,
            1);
    // small radius (0 to 1)
    private SpinnerNumberModel r2 = new SpinnerNumberModel(1,
            1,
            1000,
            1);
    private TexCoordGeneration texCoordGen = null;
    private String imageName = "texture10.jpg";


    public RunShapePanel() {
        setToolTipText("A Shape Panel!");

        addTab("dimensions",
                null,
                getDimensionPanel(),
                "dimensions");
        addTab("Shape Selector",
                null,
                getShapeSelectorPanel(),
                "Shape Selector");
        addTab("TextureCoordGenerator",
                null,
                new RunTexCoordGenerationPanel() {
                    public void run() {
                        texCoordGen = super.getValue();
                    }
                },
                "TextureCoordGenerator");
        addTab("Image Selector",
                null,
                new RunImageSelectorPanel() {
                    public void run() {
                        imageName = super.getValue();
                    }
                },
                "ImageSelector");
    }

    private Component getDimensionPanel() {
        JPanel jp = new JPanel();

        jp.setLayout(new DialogLayout());
        jp.add(new JLabel("n1"));

        jp.add(new RunSpinnerSlider(n1) {
            public void run() {
                n1.setValue(new Integer(getValue()));
                RunShapePanel.this.run();
            }
        });
        jp.add(new JLabel("n2"));

        jp.add(new RunSpinnerSlider(n2) {
            public void run() {
                n2.setValue(new Integer(getValue()));
                RunShapePanel.this.run();
            }
        });
        jp.add(new JLabel("r1"));

        jp.add(new RunSpinnerSlider(r1) {
            public void run() {
                r1.setValue(new Integer(getValue()));
                RunShapePanel.this.run();
            }
        });
        jp.add(new JLabel("r2"));

        jp.add(new RunSpinnerSlider(r2) {
            public void run() {
                r2.setValue(new Integer(getValue()));
                RunShapePanel.this.run();
            }
        });
        return jp;
    }

    private Component getShapeSelectorPanel() {
        JPanel jp = new JPanel();

        jp.setLayout(new DialogLayout());
        jp.add(new JLabel("shape"));

        jp.add(new RunComboBox(shapesFunctionInterface) {
            public void run() {
                currentShape = (Vec3FunctionInterface) getSelectedItem();
                RunShapePanel.this.run();
            }
        });

        return jp;
    }


    private static final int getValue(SpinnerNumberModel nm) {
        return ((Integer) nm.getValue()).intValue();
    }

    public Shape3D getShape3D() {
        Appearance appearance = getAppearance();
        return new PolyhedralShape3d(getGeometry(), appearance);
    }

    private Appearance getAppearance2() {
        Appearance app = new Appearance();
        //app.setTexture(getMandleTexture(w, h));
        ResourceManager rm = ResourceManager.getResourceManager();
        app.setTexture(Utils.getTexture(ImageUtils.getImage(rm.getImageFile("texture10.jpg"))));
        app.setTexCoordGeneration(new TexCoordGeneration(TexCoordGeneration.OBJECT_LINEAR,
                TexCoordGeneration.TEXTURE_COORDINATE_2));

        return app;
    }

    private Appearance getAppearance() {
        Appearance app = new Appearance();
        Color3f objColor = new Color3f(0.5f, 0.5f, 0.9f);
        Color3f black = new Color3f(0.0f, 0.0f, 0.0f);
        Color3f white = new Color3f(1.0f, 1.0f, 1.0f);
        app.setMaterial(new Material(objColor,
                black,
                objColor,
                white,
                100.0f));

        return app;
    }

    private Vec3[][] getGeometry() {


        return getGeometry(currentShape);
    }

    public Vec3[][] getGeometry(Vec3FunctionInterface tp3) {
        TriangleGeometryUtils ptg = new TriangleGeometryUtils(tp3);
        Vec3[][] triangles = ptg.getTriangles(getValue(n1),
                getValue(n2),
                getValue(r1) / 100.,
                getValue(r2) / 100.);
        return triangles;
    }


    public static void main(String[] args) {
        ClosableJFrame cf = new ClosableJFrame();
        Container c = cf.getContentPane();
        c.setLayout(new FlowLayout());
        c.add(new RunShapePanel() {
            public void run() {
                Shape3D s = getShape3D();
                System.out.println(s);
            }
        });

        cf.setSize(400, 400);
        cf.show();
    }

    public void showInFrame() {
        ClosableJFrame cf = new ClosableJFrame();
        Container c = cf.getContentPane();
        c.setLayout(new FlowLayout());
        c.add(this);

        cf.setSize(400, 400);
        cf.show();
    }

    public void mouseClicked(MouseEvent e) {
        //this.setCenterPoint(e.getX(), e.getY());
    }

    public void mouseEntered(MouseEvent e) {
    }

    public void mouseExited(MouseEvent e) {
    }

    public void mouseReleased(MouseEvent e) {

    }

    public void mousePressed(MouseEvent e) {

    }

    public void mouseDragged(MouseEvent e) {
        int m = e.getModifiers();
        //int xc = getValue(xcNm);
        //int yc = getValue(ycNm);

        //final int dx = (int) (e.getX() - xc);
        //final int dy = (int) (e.getY() - yc);

        if ((m & (Event.SHIFT_MASK)) != 0) {
            //setScale(dx, dy);
        } else if ((m & (Event.ALT_MASK)) != 0) {
            //setRotation(dx, dy);
        } else {
            //setTranslation(dx, dy);
        }
    }

    public void mouseMoved(MouseEvent e) {

    }

}
