package j3d.polyViewer;

/**
 CSE391 demo. Uses Java3D to view a list of triangles, to show
 how dynamically generated object can be controlled and displayed.
 Adapted from Sun's GearTest example, by GWH, Sept 2003, but object
 is a simple torus with structure determined by GUI controls.
 */

import com.sun.j3d.utils.applet.MainFrame;
import com.sun.j3d.utils.behaviors.vp.OrbitBehavior;
import com.sun.j3d.utils.universe.SimpleUniverse;
import com.sun.j3d.utils.universe.ViewingPlatform;
import gui.run.RunShapePanel;

import javax.media.j3d.*;
import javax.vecmath.Color3f;
import javax.vecmath.Point3d;
import javax.vecmath.Vector3f;
import java.applet.Applet;
import java.awt.*;

import j3d.utils.Utils;
import j3d.examples.checkers.CheckerFloor;

public class MainViewer extends Applet {

    private SimpleUniverse su = null;
    private RunShapePanel rsp = new RunShapePanel() {
        public void run() {
            updateShape();
        }
    };
    private BranchGroup objSplice;
    // where our thing gets spliced into scene
    private BranchGroup objBranch;
    // group containing our thing

    public static void main(String[] args) {
        new MainFrame(new MainViewer(), 400, 400);
    }


    // Set up the two frames and their contents
    public void init() {
        // create control panel in separate JFrame
        rsp.showInFrame();

        // put 3D canvas in this frame
        setLayout(new BorderLayout());
        GraphicsConfiguration config =
                SimpleUniverse.getPreferredConfiguration();
        Canvas3D c = new Canvas3D(config);
        add("Center", c);

        // Create a simple scene and attach it to the virtual universe
        BranchGroup scene = createSceneGraph();
        su = new SimpleUniverse(c);
        mouseControl(c);
        // This will move the ViewPlatform back a bit so the
        // objects in the scene can be viewed.
        su.getViewingPlatform().setNominalViewingTransform();

        su.addBranchGraph(scene);
    }


    // Create our scene graph, with lights, rotation, and object
    public BranchGroup createSceneGraph() {
        // Create the root of the branch graph
        BranchGroup objRoot = new BranchGroup();

        // Create a Transformgroup to scale all objects so they
        // appear in the scene.
        TransformGroup objScale = new TransformGroup();
        Transform3D t3d = new Transform3D();
        t3d.setScale(0.6);
        objScale.setTransform(t3d);
        objRoot.addChild(objScale);

        // Create a bounds for the background and lights
        BoundingSphere bounds =
                new BoundingSphere(new Point3d(0.0, 0.0, 0.0), 100.0);

        setBackgroundColor(bounds, objScale);
        setupGlobalLights(bounds, objScale);
        TransformGroup objTrans = setUpwritableTransformGroup(objScale);


        // first as list of triangles
        Shape3D thing = rsp.getShape3D();
        initObjectSplice();
        objBranch = new BranchGroup();
        objBranch.setCapability(BranchGroup.ALLOW_DETACH);
        objTrans.addChild(objSplice);
        objSplice.addChild(objBranch);
        objBranch.addChild(thing);

        // Create a new Behavior object that will rotate the object and
        // add it into the scene graph.
        Transform3D yAxis = new Transform3D();
        Alpha rotationAlpha = new Alpha(-1, Alpha.INCREASING_ENABLE,
                0, 0,
                8000, 0, 0,
                0, 0, 0);
        RotationInterpolator rotator =
                new RotationInterpolator(rotationAlpha, objTrans, yAxis,
                        0.0f, (float) Math.PI * 2.0f);
        rotator.setSchedulingBounds(bounds);
        objTrans.addChild(rotator);
        // todo make a view property panel that lets
        // you set your coordinate frame
        //objRoot.addChild(new CheckerFloor().getBG());
        return objRoot;
    }

    private static TransformGroup setUpwritableTransformGroup(TransformGroup objScale) {
        TransformGroup objTrans = new TransformGroup();
        objTrans.setCapability(TransformGroup.ALLOW_TRANSFORM_WRITE);
        objScale.addChild(objTrans);
        return objTrans;
    }

    private void setupGlobalLights(BoundingSphere bounds,
                                   TransformGroup objScale) {
        Color3f light1Color = new Color3f(1.0f, 1.0f, 0.9f);
        Vector3f light1Direction = new Vector3f(4.0f, -7.0f, -12.0f);
        Color3f light2Color = new Color3f(0.3f, 0.3f, 0.4f);
        Vector3f light2Direction = new Vector3f(-6.0f, -2.0f, -1.0f);
        Color3f ambientColor = new Color3f(0.1f, 0.1f, 0.1f);

        AmbientLight ambientLightNode = new AmbientLight(ambientColor);
        ambientLightNode.setInfluencingBounds(bounds);
        objScale.addChild(ambientLightNode);

        DirectionalLight light1
                = new DirectionalLight(light1Color, light1Direction);
        light1.setInfluencingBounds(bounds);
        objScale.addChild(light1);

        DirectionalLight light2 = new DirectionalLight(light2Color,
                light2Direction);
        light2.setInfluencingBounds(bounds);
        objScale.addChild(light2);
    }

    private void setBackgroundColor(BoundingSphere bounds,
                                    TransformGroup objScale) {
        Color3f bgColor = new Color3f(0.95f, 0.9f, 0.4f);
        Background bgNode = new Background(bgColor);
        bgNode.setApplicationBounds(bounds);
        objScale.addChild(bgNode);
    }

    private void mouseControl(Canvas3D c) {
        BoundingSphere bs =
                new BoundingSphere(new Point3d(0, 0, 0), 100);
        OrbitBehavior orbit =
                new OrbitBehavior(c, OrbitBehavior.REVERSE_ALL);
        orbit.setSchedulingBounds(bs);
        ViewingPlatform vp = su.getViewingPlatform();
        vp.setViewPlatformBehavior(orbit);
    }

    private void initObjectSplice() {
        objSplice = new BranchGroup();
        objSplice.setCapability(BranchGroup.ALLOW_CHILDREN_EXTEND);
        objSplice.setCapability(BranchGroup.ALLOW_CHILDREN_WRITE);
    }

    public void updateShape() {
        Shape3D newThing = rsp.getShape3D();
        BranchGroup newBG = new BranchGroup();
        // make a branchGroup for it
        newBG.addChild(newThing);
        objBranch.detach();
        // remove old one
        objBranch = newBG;
        objBranch.setCapability(BranchGroup.ALLOW_DETACH);
        objSplice.addChild(objBranch);
        // splice in new one
    }

    // when all done...
    public void destroy() {
        su.cleanup();
    }


}
